<?php
/**
*	Esta clase conecta a una base de datos mySQL y realiza las consultas devolviendo query o resultados segun corresponda.
*	Envia excepciones para conexion, seleccion de bd y consulta
*	La clase escapa los parametros de entrada
*	Sera instanciada desde el TGod
*
*	Santiago Rodriguez  
*	santi@camboya.ws
*	2009-07-21
*/

class TSQL
{
	private $link = null;
	private $query;
	private $result;
	private $index;
	
	//Conector
	private function connect()
	{
		//Se conecta a la base de datos
		$this->link = mysqli_connect(BD_SERVER, BD_USER, BD_PASS);
		if(!$this->link) throw new Exception("No se pudo conectar a la base de datos.");
		
		//Selecciona la base de datos
		$sel_db = mysqli_select_db($this->link, BD_NAME);
		if(!$sel_db) throw new Exception("No se pudo usar la base de datos.");
	}
		
	//setter de la variable query
	public function setQuery( $q )
	{
		if($this->link == null)
			$this->connect();
		$this->query = trim($q);
	}
	
	//asigna un parametro string a las querys escapando los caracteres
	public function addString($key, $value) 
	{
		$this->query = str_ireplace(":" . trim($key), sprintf("'%s'", mysqli_real_escape_string($this->link, $value)), $this->query);
	}
	
	//asigna un parametro date a las querys escapando los caracteres
	//La fecha la toma como DD-MM-YYYY y la transforma a YYYY-MM-DD
	public function addDate($key, $value) 
	{
		$el = explode("-", $value);
		$this->addString($key, $el[2]."-".$el[1]."-".$el[0]);
	}
	
	//asigna un parametro integer a las querys escapando los caracteres
	public function addInteger($key, $value) 
	{
		$this->query = str_ireplace(":" . trim($key), sprintf("%d", mysqli_real_escape_string($this->link, $value)), $this->query);
	}
	
	//asigna un parametro float a las querys escapando los caracteres
	public function addFloat($key, $value) 
	{
		$this->query = str_ireplace(":" . trim($key), sprintf("%f", mysqli_real_escape_string($this->link, $value)), $this->query);
	}
	
	
	//asigna un parametro boolean a las querys escapando los caracteres
	public function addBoolean($key, $value) 
	{
		$this->query = str_ireplace(":" . trim($key), ((isset($value) && $value == "on") ? "1":"0"), $this->query);
	}
	
	//ejecuta una consulta generica
	private function execute()
	{
		//echo $this->query;
		$this->result = mysqli_query($this->link, $this->query);
		if(!$this->result) throw new Exception("Fallo de la consulta a la base de datos.");
	}
	
	//ejecuta una consulta SELECT, SHOW, DESCRIBE o EXPLAIN, devuelve la cantidad de filas almacenadas en result
	public function executeQuery()
	{
		$this->execute();
		$this->index = 0;
		return mysqli_num_rows($this->result);
	}

	//ejecuta una consulta UPDATE, DELETE, DROP, INSERT, etc. Devuelve la cantidad de filas afectadas
	public function executeUpdate()
	{
		$this->execute();
		return mysqli_affected_rows($this->link);
	}
	//Devuelve la ID auto numerica generada en el ultimo insert
	public function lastInsertID()
	{
		return mysqli_insert_id($this->link);
	}
	
	//Genera una consulta y me devuelve el array del registro o false si no hay
	public function getRow()
	{
		return mysqli_fetch_array($this->result, MYSQLI_BOTH);
	}
	
	public function getNumFields()
	{
		return mysqli_num_fields($this->result);
	}
	
	public function reset()
	{
		return mysqli_data_seek($this->result, 0);
	}
}
?>
