<?php
/**
Clase para manejo de imagenes. 
Siempre vuelve un directorio para atras para aplicar el path, teniendo que estaria en un directorio de 'include'
De esta manera el path es relativo.

El array sizes genera thumbnails. 
El primer elemento es el tamaño ppal de la imagen.
De cada elemento, el primero es el width, el segundo es el height y el tercero flag de cropeo
	flag
	1 resizea ambos segun ancho
	2 resizea ambos segun alto
	3 ancho resize cropea alto (thumb imagenes verticales)
	4 alto resize cropea ancho (thumb imagenes horizontales)
	5 cropea ambos
	6 ajusta a canvas
Version: 14/09/2010

*/
class TImg
{
	private $path;
	private $name;
	private $ext;
	private $width;
	private $height;
	public $sizes = array();
	
	//$srcType 1: via post, 2:path subido anteriormente
	public function TImg($srcType, $src, $path, $name, $ext)
	{
		$this->path = $path . ((substr($path, -1) == "/") ? "" : "/" );
		$this->name = $name;
		$this->ext = strtoupper($ext);
		
		//Guardo la imagen segun el field
		$path = "." . $this->getFullPath();


		if(file_exists($path))
			unlink($path);

		if($srcType == 1) {
			//Si no viene via POST viene via RAW POST DATA
			if( $src == "")
			{
				$fp = fopen($path, 'wb');	
				$im =  isset($GLOBALS["HTTP_RAW_POST_DATA"])?$GLOBALS["HTTP_RAW_POST_DATA"]:file_get_contents('php://input');
				fwrite($fp, $im);
				fclose($fp);
			}
			else
			{
				move_uploaded_file($_FILES[$src]['tmp_name'], $path);
			}
		} else if($srcType == 2){
			echo $src . " - " . $path . "<br />";
			rename($src, $path);
		}
		//chmod($path, 0644);
		
		list( $width, $height ) = getimagesize( $path );
		$this->width = $width;
		$this->height = $height;
	}

	public function getWidth()
	{
		return $this->width;
	}
	
	public function getHeight()
	{
		return $this->height;
	}

	private function getFullPath()
	{
		return $this->path . $this->name . "." . $this->ext;
	}

	public function saveResized()
	{	
		foreach($this->sizes as $newSize) 
		{	
			//echo 'Attempting flag ' . $newSize[2] . '<br>';
			$this->resize("." . $this->getFullPath(), "." . $this->path . $this->name . "_" . $newSize[0] . 'x' . $newSize[1] . '_' . $newSize[2] . "." . $this->ext, $newSize[0], $newSize[1], $newSize[2]);
		}
	}
		
	private function resize($srcPath, $destPath, $newWidth, $newHeight, $flag)
	{
		list( $width, $height ) = getimagesize( $srcPath );
		
		$destX = 0;
		$destY = 0;
		
		$fromX = 0;
		$fromY = 0;
		
		switch($flag)
		{
			case 1:
				$newHeight = (int)($newWidth * $height / $width);
				$thumb = imagecreatetruecolor( $newWidth, $newHeight );
				break;
			case 2:
				$newWidth = (int)($newHeight * $width / $height);
				$thumb = imagecreatetruecolor( $newWidth, $newHeight );
				break;
			case 3:
				//resize ancho cropea alto
				$aux = (int)($width * $newHeight / $newWidth);
				$fromY = (int)(($height - $aux) / 2);
				$height = $aux;
				$thumb = imagecreatetruecolor( $newWidth, $newHeight );
				break;
			case 4:
				//resize alto cropea ancho
				$aux = (int)($height * $newWidth / $newHeight);
				$fromX = (int)(($width - $aux) / 2);
				$width = $aux;
				$thumb = imagecreatetruecolor( $newWidth, $newHeight );
				break;
			case 5:
				$fromX = (int)(($width - $newWidth) / 2);
				$fromY = (int)(($height - $newHeight) / 2);
				$width = $newWidth;
				$height = $newHeight;
				$thumb = imagecreatetruecolor( $newWidth, $newHeight );
				break;
			case 6:
				$diffW = $width - $newWidth;
				$diffH = $height - $newHeight;
				if($diffH > $diffW)
				{
					//Resizea segun alto
					$aux = $newWidth;
					$newWidth = (int)($newHeight * $width / $height);
					$destX = ( $aux - $newWidth ) / 2;
					$thumb = imagecreatetruecolor( $aux, $newHeight );
				}
				else
				{
					//Resizea segun ancho
					$aux = $newHeight;
					$newHeight = (int)($newWidth * $height / $width);
					$destY = ( $aux - $newHeight ) / 2;
					$thumb = imagecreatetruecolor( $newWidth, $aux );
				}
				$color = imagecolorallocate($thumb, 255, 255, 255);
				imagefill($thumb, 0, 0, $color);   
				break;
		}
		
		 
		if ($this->ext == "JPG" /* || $this->ext == "TIFF"*/) 
		{
			$source = imagecreatefromjpeg($srcPath);
			$doSharpen	= TRUE;
		}
		else if($this->ext == "PNG")
		{
			imagealphablending($thumb, false);
			imagesavealpha($thumb, true);
			$trans_colour = imagecolorallocatealpha($thumb, 0, 0, 0, 127);
			imagefill($thumb, 0, 0, $trans_colour);   
			$source = imagecreatefrompng($srcPath);
		}
			
		imagecopyresampled($thumb, $source, $destX, $destY, $fromX, $fromY, $newWidth, $newHeight, $width, $height );
		
		if ($doSharpen)
		{
			// Sharpen the image based on two things:
			//	(1) the difference between the original size and the final size
			//	(2) the final size
			$sharpness	= $this->findSharp($width, $newWidth);
			
			$sharpenMatrix	= array(
				array(-1, -2, -1),
				array(-2, $sharpness + 12, -2),
				array(-1, -2, -1)
			);
			$divisor		= $sharpness;
			$offset			= 0;
			imageconvolution($thumb, $sharpenMatrix, $divisor, $offset);
		}
		
		if( file_exists( $destPath ) ) unlink( $destPath );
		
		if($this->ext == "JPG"/* || $this->ext == "TIFF"*/)
			imagejpeg( $thumb, $destPath );
		else if($this->ext == "PNG")
			imagepng( $thumb, $destPath );
		
		chmod($destPath, 0644);
	}
	
	public function getDPI() {

		$filename = "." . $this->getFullPath();
		// open the file and read first 20 bytes.
		$a = fopen($filename,'r');
		$string = fread($a,20);
		fclose($a);

		// get the value of byte 14th up to 18th
		$data = bin2hex(substr($string,14,4));
		$x = substr($data,0,4);
		$y = substr($data,4,4);
		return array(hexdec($x),hexdec($y));
	}
	
	private function findSharp($orig, $final) // function from Ryan Rud (http://adryrun.com)
	{
			$final	= $final * (750.0 / $orig);
			$a		= 52;
			$b		= -0.27810650887573124;
			$c		= .00047337278106508946;
			
			$result = $a + $b * $final + $c * $final * $final;
			
			return max(round($result), 0);
	} // findSharp()
	
}
?>