<?php
/**
*	Class de validadores. Se instancia TValidator y desde esta se van cargando los validadores necesarios
* 	Se definien de antemano los mensajes de error.
*	Sera Instanciada desde el TGod
*
*	Santiago Rodriguez  
*	santi@camboya.ws
*	2009-07-21
*/

//Parametro para Method Array
define("VAL_NONE", 0);
define("VAL_POST", 1);
define("VAL_GET", 2);

class TValidator
{
	private $validators;			//Array con validadores
	private $methodArray;		//Array segun el metodo de las variables POST o GET
	private $errorMsgs;			//Array con los mensajes de error
	
	public function TValidator($methodArray)
	{
		$this->validators = array();
		$this->errorMsgs = array();
		$this->methodArray = $methodArray == VAL_POST ? $_POST : $_GET;
	
		$this->save = array();
	}
	
	// Recorre todos los validadores instanciados, los valida y en caso negativo carga los mensajes de error correspondientes
	public function validate()
	{
		$valid = true;
		
		foreach($this->validators as $validator) 
		{
			if( !$validator->validate() )
			{
				$valid = false;
				$this->errorMsgs[$validator->getLabel()] = $validator->getErrorMsg();
			}
		}
		
		return $valid;
	}
	
	//devuelve el primer error
	public function getFirstError()
	{
		foreach($this->errorMsgs as $key=>$value)
			return array($key, $value);
			
		return array("", "");
	}
	
	//Imprime los errores del formulario en un ul con nombre de clase $nomClass
	public function printErrors($nomClass)
	{
		echo "<ul class=\"" . $nomClass . "\">";
		foreach($this->errorMsgs as $value) 
			echo "<li>" . $value . "</li>";
		echo "</ul>";
	}
	
	//Agerga validador requerido
	public function addMD5Validator($field, $label)
	{
		if(VAL_MD5)
			$this->validators[] = new TMD5Validator($this->methodArray[$field], $label, $this->methodArray);
	}
	
	//Agerga validador requerido
	public function addRequiredValidator($field, $label)	
	{
		$this->validators[] = new TRequiredValidator($this->methodArray[$field], $label);
	}
	
	//clase privada para usar dentro de los siguientes adds cuando el field es requerido
	//Agrega el required validator de ser necesario y responde si puede agregar el otro
	private function canAdd($field, $label, $required)
	{
		$reqVal = new TRequiredValidator($this->methodArray[$field], $label);
		$valid = $reqVal->validate();

		if(!$valid && $required)
			$this->validators[] = $reqVal;

		return $valid;
	}

	//clase privada para usar dentro de los siguientes adds cuando el field es requerido y es un file
	//Agrega el required validator de ser necesario y responde si puede agregar el otro
	private function canAddFile($field, $label, $required)
	{
		$reqVal = new TRequiredValidator($_FILES[$field]['name'], $label);
		$valid = $reqVal->validate();
		
		if(!$valid && $required)
			$this->validators[] = $reqVal;

		return $valid;
	}
	
	//Agerga validador solo numeros
	public function addNumberValidator($field, $label, $min, $max, $required)
	{
		if($this->canAdd($field, $label, $required)) 
			$this->validators[] = new TNumberValidator($this->methodArray[$field], $label, $min, $max);
	}
	
	//Agerga validador string length
	public function addStrLengthValidator($field, $label, $min, $max, $required)
	{
		if($this->canAdd($field, $label, $required)) 
			$this->validators[] = new TStrLengthValidator($this->methodArray[$field], $label, $min, $max);
	}
	
	//Agerga validador email
	public function addEmailValidator($field, $label, $required)
	{
		if($this->canAdd($field, $label, $required)) 
			$this->validators[] = new TEmailValidator($this->methodArray[$field], $label);
	}
	
	//Agerga validador date
	public function addDateValidator($field, $label, $required)
	{
		if($this->canAdd($field, $label, $required)) 
			$this->validators[] = new TDateValidator($this->methodArray[$field], $label);
	}
	
	//Agerga validador pass confirm
	public function addPassConfirmValidator($field, $label, $fieldConfirm, $required)
	{
		if($this->canAdd($field, $label, $required)) 
			$this->validators[] = new TPassConfirmValidator($this->methodArray[$field], $label, $this->methodArray[$fieldConfirm]);
	}
	
	//Agerga validador de extension de archivo
	public function addFileExtensionValidator($field, $label, $arrExt, $required)
	{
		if($this->canAddFile($field, $label, $required)) 
			$this->validators[] = new TFileValidator($_FILES[$field]['name'], $label, $arrExt);
	}
	
	//Agerga validador enum
	public function addEnumValidator($field, $label, $arr, $required)
	{
		if($this->canAdd($field, $label, $required)) 
			$this->validators[] = new TEnumValidator($this->methodArray[$field], $label, $arr);
	}
	
	//Agrega validator HTTP_RAW_POST_DATA para archivos binarios
	public function addHttpRawPostData()
	{
		$this->validators[] = new THttpRawPostDataValidator(isset($GLOBALS["HTTP_RAW_POST_DATA"])?$GLOBALS["HTTP_RAW_POST_DATA"]:file_get_contents('php://input'), "file");
	}
	
	//Agrega validador de duplicados
	public function addDuplicateValidator($value, $label, $table)
	{	
		$this->validators[] = new TDuplicateValidator($this->methodArray[$value], $label, $table);
	}
	public function addJpgValidator($field, $label, $required)
	{
		if($this->canAddFile($field, $label, $required)) 
			$this->validators[] = new TFileValidator($_FILES[$field]['name'], $label, "JPG");
	}

	public function addCodeValidator($field, $label, $required)
	{
			$this->validators[] = new TCodeValidator($this->methodArray[$field], $label);
	}
	
}
//Clase abstracta para validadores 
abstract class TAbstractValidator
{
	protected $value;
	protected $label;
	
	public function TAbstractValidator($value, $label)
	{
		$this->value = $value;
		$this->label = $label;
	}
	
	//Metodos abstractos que deben implementar las clases hijas
	abstract protected function getMsg();
	abstract public function validate();
	
	public function getLabel()
	{
		return $this->label;
	}
	
	public function getErrorMsg()
	{
		return str_ireplace(":label", sprintf("%s", $this->label), $this->getMsg());
	}
}
//Validador MD5
class TMD5Validator extends TAbstractValidator 
{
	private $methodArr;
	
	public function TMD5Validator($value, $label, $methodArr)
	{
		parent::TAbstractValidator($value, $label);
		$this->methodArr = $methodArr;
		ksort($this->methodArr);
	}
	
	public function validate()
	{
		$str = "";
		foreach ($this->methodArr as $key => $val)
			if($key != "validation")
				$str .= stripslashes($val) . "|";
		
		$str .= VAL_KEY;

		//echo "-".$str."-";
		//echo "-".md5($str)."-";
		//echo "-".$this->value."-";
		

		return ($this->value == md5(utf8_encode($str)));
	}
	
	protected function getMsg()
	{
		return MD5_MSG;
	}
}

//Validador requerido
class TRequiredValidator extends TAbstractValidator 
{
	public function validate()
	{
		return (isset($this->value) && strlen($this->value) > 0);
	}
	
	protected function getMsg()
	{
		return REQUIRED_MSG;
	}
}

//Validador only numbers
class TNumberValidator extends TAbstractValidator 
{
	private $min;
	private $max;
	
	public function TNumberValidator($value, $label, $min, $max)
	{
		parent::TAbstractValidator($value, $label);
		$this->min = $min;
		$this->max = $max;
	}
	
	public function validate()
	{
		if(!is_numeric($this->value)) return false;
		if($this->min != null && $this->value < $this->min) return false;
		if($this->max != null && $this->value > $this->max) return false;
		return true;
	}
	
	protected function getMsg()
	{
		return NUMBER_MSG;
	}
}

//Validador string length
class TStrLengthValidator extends TAbstractValidator 
{
	private $min;
	private $max;
	
	public function TStrLengthValidator($value, $label, $min, $max)
	{
		parent::TAbstractValidator($value, $label);
		$this->min = $min;
		$this->max = $max;
	}
	
	public function validate()
	{
		if(strlen($this->value) < $this->min) return false;
		if(strlen($this->value) > $this->max) return false;
		return true;
	}
	
	protected function getMsg()
	{
		return STR_LENGTH_MSG;
	}
}

//Validador email
class TEmailValidator extends TAbstractValidator 
{
	public function validate()
	{
		return (filter_var($this->value, FILTER_VALIDATE_EMAIL));
	}
	
	protected function getMsg()
	{
		return EMAIL_MSG;
	}
}

//Valida fecha DD-MM-YYYY
class TDateValidator extends TAbstractValidator 
{
	public function validate()
	{
		$day = (int)substr($this->value, 0, 2);
		$month = (int)substr($this->value, 3, 2);
		$year = (int)substr($this->value, 6, 4);
	
		return (checkdate($month, $day, $year ));
	}
	
	protected function getMsg()
	{
		return DATE_MSG;
	}
}

//Validador passConfirm
class TPassConfirmValidator extends TAbstractValidator 
{
	private $pass;
	
	public function TPassConfirmValidator($value, $label, $pass)
	{
		parent::TAbstractValidator($value, $label);
		$this->pass = $pass;
	}
	
	public function validate()
	{
		return ( $this->pass == $this->value );
	}
	
	protected function getMsg()
	{
		return PASS_CONFIRM_MSG;
	}
}

//Validador de archivo segun la extension
class TFileValidator extends TAbstractValidator 
{
	private $arr;
	
	public function TFileValidator($value, $label, $arrExt)
	{
		parent::TAbstractValidator($value, $label);
		$this->arr = $arrExt;
	}
	
	public function validate()
	{
			if(strtoupper($this->arr) == strtoupper(substr($this->value, -strlen($this->arr))))
				return true;
		return false;
	}
	
	protected function getMsg()
	{
		return FILE_MSG;
	}
}

//Validador de un valor que debe estar en un array
class TEnumValidator extends TAbstractValidator 
{
	private $arr;
	
	public function TEnumValidator($value, $label, $arr)
	{
		parent::TAbstractValidator($value, $label);
		$this->arr = $arr;
	}
	
	public function validate()
	{
		foreach($this->arr as $value) 
			if($this->value == $value)
				return true;
				
		return false;
	}
	
	protected function getMsg()
	{
		return ENUM_MSG;
	}
}

//Validador de duplicados en la tabla
class TDuplicateValidator extends TAbstractValidator
{
	private $table;
	private $valSQL;
	
	public function TDuplicateValidator($value, $label, $table)	{
		parent::TAbstractValidator($value, $label);
		$this->table = $table;
		$this->valSQL = new TSQL();
	}
	
	public function validate()	{
		try {
				$this->valSQL->setQuery("SELECT * FROM " . $this->table . " WHERE " . $this->label . "= :" . $this->label . ";");
				$this->valSQL->addString($this->label, $this->value);
				if ($this->valSQL->executeQuery()) {
					return false;
				}
				return true;
		} 
		catch (Exception $e) {
			return false;
		}
	}
	protected function getMsg()
	{
		return DUPLICATED_MSG;
	}
}


//Validador HTTP_RAW_POST_DATA
class THttpRawPostDataValidator extends TAbstractValidator 
{
	public function validate()
	{
		return ($this->value != null);
	}
	
	protected function getMsg()
	{
		return HTTP_RAW_POST_DATA_MSG;
	}
}

?>